/*
 * Copyright (C) Philipp Meier 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.praeber.humigraf;

import ch.praeber.humigraf.model.DataManager;
import ch.praeber.humigraf.system.Controller;
import ch.praeber.humigraf.system.Fahrenheit;
import ch.praeber.humigraf.system.ThreadErrors;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.Observable;
import java.util.Observer;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import org.jfree.chart.*;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.general.DatasetGroup;
import org.jfree.ui.RefineryUtilities;

/**
 * Main UI and Application entrance point. Initalises main GUI
 *
 * @author Meier Philipp
 *
 */
public class AnalyticsGUI extends JFrame implements Observer, ChartMouseListener {

    //Menu Elements
    private JMenuBar menuBar;
    private JMenu menuFile;
    private JMenuItem miFileNew;
    private JMenuItem miFileOpen;
    private JMenuItem miFileSave;
    private JMenuItem miFileExport;
    private JMenuItem miFileExit;
    private JMenu menuHelp;
    private JMenuItem miHelpHelp;
    private JMenuItem miHelpManual;
    private JMenuItem miHelpAbout;
    private JMenu menuDevice;
    private JMenuItem miDeviceImport;
    private JMenuItem miDeviceLive;
    private JMenuItem miDeviceSettings;
    private JMenuItem miDeviceDelete;
    //Popup Menu
    private JMenuItem miDefaultView;
    //Chart Elements
    private BorderLayout lay;
    private JFreeChart chart;
    private ChartPanel chartPanel;
    private JPanel uberPanel;
    private JPopupMenu popup;
    private JSlider slider;
    // Variables and Windows
    private static final String version = "1.0.0";
    private static final ResourceBundle languages =
            ResourceBundle.getBundle("ch/praeber/humigraf/resources/languages/Languages");
    private DataManager datamanager;
    private Controller controller;
    private JFileChooser fileChooser;
    //TODO Remeber last save & load place
    //TODO import data twice does nothing.
    private Configuration configuration;
    private JPanel values;
    private JLabel temp;
    private JLabel valtemp;
    private JLabel hum;
    private JLabel valhum;
    private JLabel dew;
    private JLabel valdew;

    /**
     * Initialises an instance of AnalyticsGUI.
     *
     * @param title Title of Frame
     */
    public AnalyticsGUI(String title) {
        super(title);
        //Locale.setDefault(Locale.US);
        //Set Native Look & Feel        
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            e.printStackTrace();
        }

        //Set Icon
        final ImageIcon windowIcon = new ImageIcon(
                AnalyticsGUI.class.getResource("resources/icon.png"));
        setIconImage(windowIcon.getImage());
        configuration = Configuration.getInstance();

        initialise();
        setupListeners();

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        pack();
        RefineryUtilities.centerFrameOnScreen(this);
        setVisible(true);
    }

    /**
     * Initialises Buttons, Panels and constructs the Window Plus it intialises
     * the Datasets
     */
    private void initialise() {
        // Menubar
        menuBar = new JMenuBar();

        //Menu File
        menuFile = new JMenu(languages.getString("FILE_MENU"));
        miFileNew = new JMenuItem(languages.getString("NEW_MENUITEM"));
        miFileOpen = new JMenuItem(languages.getString("OPEN_MENUITEM"));
        miFileSave = new JMenuItem(languages.getString("SAVE_MENUITEM"));
        miFileExport = new JMenuItem(languages.getString("EXPORT_MENUITEM"));
        miFileExit = new JMenuItem(languages.getString("EXIT_MENUITEM"));
        menuFile.add(miFileNew);
        menuFile.add(miFileOpen);
        menuFile.add(miFileExport);
        menuFile.add(miFileSave);
        menuFile.add(miFileExit);

        //Menu Device
        menuDevice = new JMenu(languages.getString("HUMIGRAF_MENU"));
        miDeviceImport = new JMenuItem(languages.getString("IMPORT DATA_MENUITEM"));
        miDeviceLive = new JMenuItem(languages.getString("LIVE TRACKING_MENUITEM"));
        miDeviceSettings = new JMenuItem(languages.getString("SETTINGS_MENUITEM"));
        miDeviceDelete = new JMenuItem(languages.getString("DELETE DEVICE MEMORY_MENUITEM"));
        menuDevice.add(miDeviceLive);
        menuDevice.add(miDeviceImport);
        menuDevice.add(miDeviceDelete);
        menuDevice.add(miDeviceSettings);

        //Menu Help
        menuHelp = new JMenu(languages.getString("HELP_MENU"));
        miHelpHelp = new JMenuItem(languages.getString("HELP_MENUITEM"));
        miHelpManual = new JMenuItem(languages.getString("DOWNLOAD DEVICE MANUAL_MENUITEM"));
        miHelpAbout = new JMenuItem(languages.getString("ABOUT_MENUITEM"));
        menuHelp.add(miHelpHelp);
        menuHelp.add(miHelpManual);
        menuHelp.add(miHelpAbout);

        //Slider
        //slider = new JSlider();

        menuBar.add(menuFile);
        menuBar.add(menuDevice);
        menuBar.add(menuHelp);
        setJMenuBar(menuBar);

        //Datamanager
        datamanager = new DataManager();
        datamanager.addObserver(this);
        //Create Controller
        controller = new Controller(datamanager);

        //Chart and Panels initialisation        
        chart = HumigrafChartFactory.createChart(datamanager.getChartTemp(), datamanager.getChartHum());
        chartPanel = new ChartPanel(chart, false, false, false, false, true);
        chartPanel.setMouseZoomable(false);
        chartPanel.addChartMouseListener(this);
        popup = chartPanel.getPopupMenu();
        chartPanel.setPopupMenu(null);
        miDefaultView = new JMenuItem(languages.getString("AUTO_SCALE"));
        popup.add(miDefaultView);
        //Todo fix error import then live tracking

        //Label panel for values display
        values = new JPanel();
        values.setBackground(Color.WHITE);
        temp = new JLabel(languages.getString("TEMPERATURE") + ": ");
        hum = new JLabel(languages.getString("HUMIDITY") + ": ");
        dew = new JLabel(languages.getString("DEW POINT") + ": ");
        valtemp = new JLabel();
        valhum = new JLabel();
        valdew = new JLabel();
        values.add(temp);
        values.add(valtemp);
        values.add(hum);
        values.add(valhum);
        values.add(dew);
        values.add(valdew);



        //Initalise containing Panel
        uberPanel = new JPanel();
        lay = new BorderLayout();
        uberPanel.setLayout(lay);
        uberPanel.add(chartPanel, BorderLayout.CENTER);
        uberPanel.add(values, BorderLayout.NORTH);
        setContentPane(uberPanel);



    }

    /**
     * Sets up the Listeners to all Menu Points
     */
    private void setupListeners() {
        //File -> New
        miFileNew.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                newFile();
            }
        });

        // File -> Open
        miFileOpen.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                openFile();
            }
        });

        // File -> Save
        miFileSave.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                saveFile();
            }
        });

        //File -> Export
        miFileExport.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                openExportDialog();
            }
        });


        // File -> Exit
        miFileExit.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                exitProcedure();
            }
        });

        //Device - Import Data
        miDeviceImport.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                openImportDialog();
            }
        });


        //Device - Start Live Tracking
        miDeviceLive.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                openTrackingDialog();
            }
        });



        //Device - Configure
        miDeviceSettings.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                openConfigureDialog();
            }
        });

        //Device - Delete Device Memory
        miDeviceDelete.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                EreaseDeviceMemory();
            }
        });


        // Help -> Help
        miHelpHelp.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {

                openHelpDialog();

            }
        });


        // Help -> Device Manual
        miHelpManual.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {

                GetManualDialog();

            }
        });



        // Help -> About
        miHelpAbout.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {

                openAboutDialog();

            }
        });

        // Popup Menu
        miDefaultView.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                XYPlot plot = (XYPlot) chartPanel.getChart().getPlot();
                //plot.clearRangeAxes();
                plot.getRangeAxis(0).setRange(HumigrafChartFactory.getTEMPERATURERANGE());
                plot.getRangeAxis(1).setRange(HumigrafChartFactory.getHUMIDITYRANGE());
                plot.getDomainAxis().setAutoRange(true);

                //chartPanel.zoomOutRange(10, 10);
                //chartPanel.zoomOutDomain(10, 10);

            }
        });

        // *** Window Listeners ***

        // Custom close operation
        addWindowListener(new WindowAdapter() {

            @Override
            public void windowClosing(WindowEvent e) {
                exitProcedure();
            }
        });





    }

    /**
     * Initialised when clicking File->new
     */
    public void newFile() {
        if (!controller.isSaved()) {
            int response = JOptionPane.showConfirmDialog(this,
                    languages.getString("WOULD YOU LIKE TO SAVE THE IMPORTED DATA?"),
                    languages.getString("CONFIRM EXIT WITHOUT SAVING"), JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE);
            // Response value: Yes = 0, No = 1, Cancel = 2
            if (response == JOptionPane.OK_OPTION) {
                if (controller.isThreadActive()) {
                    controller.stopThread();
                    //setLiveTracking(false);
                    //setAbilityToSave(true);
                }
                while (!saveFile()) {
                    JOptionPane.showMessageDialog(this,
                            languages.getString("UNABLE TO SAVE FILE.PLEASE RETRY!"), languages.getString("FILE SAVING ERROR"),
                            JOptionPane.ERROR_MESSAGE);
                }
            }
        }
        controller.resetChart();
    }

    /**
     * Exit the Window asks if the user would like to save any unsaved Data
     * first.
     *
     */
    private void exitProcedure() {
        if (!controller.isSaved()) {
            int response = JOptionPane.showConfirmDialog(this,
                    languages.getString("WOULD YOU LIKE TO SAVE THE IMPORTED DATA?"),
                    languages.getString("CONFIRM EXIT WITHOUT SAVING"),
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE);
            // Response value: Yes = 0, No = 1, Cancel = 2
            if (response == JOptionPane.OK_OPTION) {
                if (controller.isThreadActive()) {
                    controller.stopThread();
                    setLiveTracking(false);
                }

                while (!saveFile()) {
                    JOptionPane.showMessageDialog(this,
                            languages.getString(
                            "UNABLE TO SAVE FILE.PLEASE RETRY!"),
                            languages.getString("FILE SAVING ERROR"),
                            JOptionPane.ERROR_MESSAGE);
                }
            }
        }
        Configuration.saveSettings();
        System.exit(0);
    }

    /**
     * Open a file
     *
     *
     */
    public void openFile() {
        String fileName = null;
        // Set up the filter for files
        RaeberFileFilter fileFilter = new RaeberFileFilter(
                RaeberFileFilter.P_RAEBER_EXT);
        fileChooser = new JFileChooser(Configuration.getLoadlocation());

        fileChooser.setFileFilter(fileFilter);

        // Open chooser dialog
        int returnVal = fileChooser.showOpenDialog(this);

        if (returnVal == JFileChooser.CANCEL_OPTION) {
            fileName = "-1";
        } else if (returnVal == JFileChooser.APPROVE_OPTION) {
            fileName = fileChooser.getSelectedFile().getAbsolutePath();
            //TODO Validate Filename
        } else {
            JOptionPane.showMessageDialog(this,
                    languages.getString("FILE FORMAT NOT SUPPORTED! "),
                    languages.getString("OPEN FILE ERROR"),
                    JOptionPane.ERROR_MESSAGE);
        }
        newFile();
        int returnvalue = controller.loadFile(fileName);
        if (returnvalue == -1) {
            JOptionPane.showMessageDialog(this,
                    languages.getString("FILE FORMAT NOT SUPPORTED! "),
                    languages.getString("OPEN FILE ERROR"),
                    JOptionPane.ERROR_MESSAGE);
        } else if (returnvalue == -2) {
            JOptionPane.showMessageDialog(this,
                    languages.getString("FILE FORMAT NOT SUPPORTED!"),
                    languages.getString("OPEN FILE ERROR"),
                    JOptionPane.ERROR_MESSAGE);
        }
        if (!"-1".equals(fileName)) {
            Configuration.setLoadlocation(
                    fileChooser.getSelectedFile().getParent());
        }
    }

    /**
     * Save the captured Data to a properitary file format
     *
     * @return true if Saveing the Data has been sucessfull, false if not.
     */
    private boolean saveFile() {
        boolean result;
        fileChooser = new JFileChooser();
        File file = null;
        //Adds the diffrent Types of Files which are possible to Export
        fileChooser.setAcceptAllFileFilterUsed(false);
        fileChooser.addChoosableFileFilter(
                new RaeberFileFilter(RaeberFileFilter.P_RAEBER_EXT));


        int option = fileChooser.showSaveDialog(uberPanel);
        if (option == JFileChooser.APPROVE_OPTION) {
            if (fileChooser.getSelectedFile() != null) {
                file = fileChooser.getSelectedFile();
            }
        }

        //check if file not null
        if (file == null) {
            result = true; //Avoids Error if no file has been choosen (Abort)
        } else {
            //check if correct file extension           
            file = validateFileExtension(file, fileChooser.getFileFilter());
            Configuration.setSavelocation(file.getParent());
            //file.getPath()
            //save file
            if (!controller.saveToFile(file)) {
                JOptionPane.showMessageDialog(this,
                        languages.getString("UNABLE TO SAVE FILE TO:") + file.toString(), languages.getString("FILE SAVING ERROR"),
                        JOptionPane.ERROR_MESSAGE);
                result = false;
            } else {
                result = true;
                controller.setSaved(result);
            }
            //JOptionPane.showMessageDialog(rootPane, "File successfully saved to " + file.toString());
        }
        return result;
    }

    /**
     * Exports the Chart to a certain file
     *
     * @return true if it worked correctely or users aborted & false if it went
     * wrong
     */
    private boolean ExportFile() {
        //only possible if there is any data
        RaeberFileFilter png = new RaeberFileFilter("png");
        RaeberFileFilter csv = new RaeberFileFilter("csv");
        fileChooser = new JFileChooser(Configuration.getSavelocation());
        File file = null;
        //Adds the diffrent Types of Files which are possible to Export
        fileChooser.setAcceptAllFileFilterUsed(false);
        fileChooser.addChoosableFileFilter(csv);
        fileChooser.addChoosableFileFilter(png);

        int option = fileChooser.showDialog(uberPanel, languages.getString("EXPORT"));
        if (option == JFileChooser.APPROVE_OPTION) {
            if (fileChooser.getSelectedFile() != null) {
                file = fileChooser.getSelectedFile();
            }
        }
        if (file == null) {
            return true;
        } else {
            //Checks if extension is ok
            file = validateFileExtension(file, fileChooser.getFileFilter());
            if (fileChooser.getFileFilter().equals(png)) {
                boolean success;
                try {                    //Export Chart to File // Maybe add posibility to change the size of the chart
                    ChartUtilities.saveChartAsPNG(file, chart,
                            chartPanel.getWidth(), chartPanel.getHeight());
                    success = true;
                } catch (IOException ex) {
                    Logger.getLogger(AnalyticsGUI.class.getName()).log(Level.SEVERE, null, ex);
                    success = false;
                }
                if (success) {
                    //  JOptionPane.showMessageDialog(rootPane, "Data succesfully exported to: "+ file.toString());
                }
            } else if (fileChooser.getFileFilter().equals(csv)) {
                boolean success = controller.exportCSV(file);
                if (success) {
                    //  JOptionPane.showMessageDialog(rootPane, "Data succesfully exported to: "+ file.toString());
                }
            }
        }
        return true;
    }

    /**
     * Open help dialog
     */
    public void openHelpDialog() {
        JOptionPane.showMessageDialog(this, languages.getString("HELP- IMPORT DATA FROM HUMIGRAF:")
                + languages.getString("LOADING DATA FROM HUMIGRAF IS EASY. JUST GO TO THE MENU HUMIGRAF")
                + languages.getString("AND SELECT THE SUBMENU 'IMPORT DATA' AND FOLLOW THE INSTRUCTIONS THERE.")
                + languages.getString("- SAVE DATA:TO SAVE YOUR IMPORTED RECORDS")
                + languages.getString(" GO TO 'FILE' AND SELECT 'SAVE...'.")
                + languages.getString("- LIVE TRACKING:TO TRACK DATA WHILE THE DEVICE")
                + languages.getString(" IS ATTACHED TO YOUR COMPUTERGO TO THE MENU 'HUMIGRAF'AND SELECT 'LIVE TRACKING")
                + languages.getString(" AND FOLLOW THE INSTRUCTIONS THERE.")
                + languages.getString("- STOP LIVE TRACKING:TO STOP LIVE TRACKING,GO TO ")
                + languages.getString("'HUMIGRAF' AND SELECT THE SUBMENU 'LIVE TRACKING'AND PRESS THE 'STOP LIVE TRACKING' BUTTON.")
                + "\n\n"
                + languages.getString("- EXPORT DATA:TO EXPORT YOUR IMPORTED DATA TO A SPREADSHEET")
                + languages.getString(" OR WORD PROCESSING SOFTWARE, GO TO FILE THEN SELECT EXPORT...")
                + languages.getString("TO EXPORT, A PICTURE OF THE GRAPH, SELECT PNG OTHERWISE SELECT CSV")
                + languages.getString("AND CHOOSE A FILENAME."),
                languages.getString("HELP"), JOptionPane.INFORMATION_MESSAGE);
    }

    /**
     * Open about dialog
     */
    public void openAboutDialog() {
        JOptionPane.showMessageDialog(this, languages.getString("P-RAEBER HUMIGRAF") + "\n" + languages.getString("VERSION")
                + version + "\nCopyright 2011 (C) Paul Räber\n"
                + "\n" + languages.getString("P-RAEBER HUMIGRAF HELPS YOU TO ANALYZE THE DATA RECORDED BY A")
                + "\n" + languages.getString("P-RAEBER THERMO HYGROGRAPH. IT OFFERS LIVE TRACKING OF TEMPERATURE,")
                + "\n" + languages.getString("HUMIDITY AND CALCULATION OF THE DEW POINT. VISUALLY MANAGE")
                + "\n" + languages.getString("AND IMPORT YOUR RECORDS AND EXPORT THEM TO FURTHER USE AS")
                + "\n" + languages.getString("PICTURE (PNG) OR WITH SPREADSHEET SOFTWARE (CSV)."), languages.getString("ABOUT"),
                JOptionPane.INFORMATION_MESSAGE,
                new ImageIcon(
                AnalyticsGUI.class.getResource("resources/logo.png")));
    }

    /**
     * Export your Chart to / PNG / PDF or CSV
     *
     * @return true if it sucessfully has been exported, false if not
     */
    public boolean openExportDialog() {
        return ExportFile();
    }

    /**
     * Open the ImportDialog
     *
     */
    private void openImportDialog() {
        ImportDialog dial = new ImportDialog(this, controller);
    }

    /**
     * Opens the Configure Dial
     *
     */
    private void openConfigureDialog() {
        if (controller.getIntervall() < 0) {
            JOptionPane.showConfirmDialog(this, languages.getString("NO CONNECTION TO DEVICE.")
                    + "\n" + languages.getString("PLEASE PLUG-IN YOUR HUMIGRAF AND TRY AGAIN."),
                    languages.getString("NO CONNECTION TO HUMIGRAF"), JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE);
        } else {
            DeviceSettingsDialog conf = new DeviceSettingsDialog(this, controller);

        }
    }

    /**
     * Open Tracking dialog *
     */
    private void openTrackingDialog() {
        TrackingDialog dia = new TrackingDialog(this, controller);
    }

    /**
     * Checks if the file name ends with the extension of the given filter. It
     * autocorrects the filename
     *
     * @param file the file which needs to be checked
     * @param filter Filechooser FileFilter, used for file extension
     * @return
     */
    private File validateFileExtension(final File file,
            final javax.swing.filechooser.FileFilter filter) {
        if (filter.accept(file)) {
            return file;
        }
        // remove wrong file extension if any
        String fileName = file.getName();
        final int index = fileName.lastIndexOf(".");
        if (index > 0) {
            fileName = fileName.substring(0, index);
        }

        final String extension = filter.toString();
        final String newFileName = fileName + "." + extension;

        final File newFile = new File(file.getParent(), newFileName);

        return newFile;
    }

    /**
     * Opens Windows to erease device memory.
     */
    public void EreaseDeviceMemory() {
        //TODO Ask if user wants to import data
        int response = 0;
        if (!controller.isSaved()) {
            response = JOptionPane.showConfirmDialog(this,
                    languages.getString("WOULD YOU LIKE TO SAVE YOUR DATA BEFORE")
                    + languages.getString(" YOU ERASE THE RECORDS ON YOUR DEVICE?"),
                    languages.getString("SAVE IMPORTED RECORDS FROM HUMIGRAF?"), JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE);
            // Response value: Yes = 0, No = 1, Cancel = 2
            if (response == JOptionPane.YES_OPTION) {
                saveFile();
            }
        }
        response = JOptionPane.showConfirmDialog(this,
                languages.getString("WOULD YOU LIKE TO ERASE ALL RECORDS FROM YOUR THERMO HYGROGRAPH?")
                + "\n" + languages.getString("THE RECORDS CANNOT BE RESTORED AFTERWARDS!"),
                languages.getString("REMOVE ALL RECORDS FROM HUMIGRAF?"), JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE);
        // Response value: Yes = 0, No = 1, Cancel = 2
        if (response == JOptionPane.YES_OPTION) {
            //Remove all Data
            controller.formatDevice();
        }
    }

    /**
     * Opens a Manual Dialog
     */
    public void GetManualDialog() {
        boolean success = OpenBrowser.OpeninBrowser(
                "http://www.p-raeber.ch/Hygrograph_D/Downloads.htm");
    }

    /**
     * Enables / Disables menu items when live tracking is off / on
     *
     * @param live Status of live tracking
     */
    public void setLiveTracking(boolean live) {
        miFileNew.setEnabled(!live);
        miFileOpen.setEnabled(!live);
        miFileSave.setEnabled(!live);
        miDeviceImport.setEnabled(!live);
        miDeviceSettings.setEnabled(!live);
        miDeviceDelete.setEnabled(!live);
        miFileExport.setEnabled(!live);
    }

    @Override
    public void update(Observable o, Object arg) {
        ThreadErrors error = datamanager.getError();
        if (error == ThreadErrors.HUMIGRAFCONNECTIONERROR) {
            JOptionPane.showConfirmDialog(this, languages.getString("NO CONNECTION TO DEVICE.")
                    + "\n" + languages.getString("PLEASE PLUG-IN YOUR HUMIGRAF AND TRY AGAIN."),
                    languages.getString("NO CONNECTION TO HUMIGRAF"), JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE);
            setLiveTracking(false);
        } else if (error == ThreadErrors.HUMIGRAFINTERVALLERROR) {
            JOptionPane.showConfirmDialog(this, languages.getString("NO CONNECTION TO DEVICE.")
                    + languages.getString(" RECORD INTERVAL TIME UNKNOWN.")
                    + "\n" + languages.getString("PLEASE PLUG-IN YOUR HUMIGRAF AND TRY AGAIN."),
                    languages.getString("INTERVAL UNKNOWN."),
                    JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE);
            setLiveTracking(false);

        } else if (error == ThreadErrors.WAITERROR) {
            JOptionPane.showConfirmDialog(this, languages.getString("ERROR WHILE WAITING FOR DATA")
                    + "\n" + languages.getString("PLEASE TRY AGAIN."), languages.getString("INTERVAL ERROR"),
                    JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE);
            setLiveTracking(false);
        } else {
            chartPanel.setMouseWheelEnabled(true);
            chartPanel.setMouseZoomable(true, true);
            chartPanel.setPopupMenu(popup);
        }
    }

    @Override
    public void chartMouseClicked(ChartMouseEvent event) {
        //System.out.println(event.getEntity().getToolTipText());
        String tooltip = event.getEntity().getToolTipText();
        if (tooltip != null) {
            String[] split = tooltip.split(" ");
            int itemnr = Integer.parseInt(split[split.length - 1]);
            //System.out.println(itemnr);
            DecimalFormat df = new DecimalFormat("0.0");

            valtemp.setText(df.format(
                    datamanager.getChartTemp().getY(0, itemnr))
                    + " " + Fahrenheit.getSIUnit());

            valhum.setText(df.format(
                    datamanager.getChartHum().getY(0, itemnr)) + " %");

            valdew.setText(df.format(
                    datamanager.getChartTemp().getY(3, itemnr))
                    + " " + Fahrenheit.getSIUnit());
        } else {
            valtemp.setText("");
            valhum.setText("");
            valdew.setText("");
        }
    }

    @Override
    public void chartMouseMoved(ChartMouseEvent event) {
    }
}
