/*
 * Copyright (C) Philipp Meier 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.praeber.humigraf;

import ch.praeber.humigraf.system.Controller;
import ch.praeber.humigraf.system.Fahrenheit;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerListModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.Border;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * DeviceSettingsDialog Configuration Dialog to graphicaly adjust settings of
 * Humigraf
 *
 * @author Meier Philipp
 */
public class DeviceSettingsDialog extends JDialog {

    private static final ResourceBundle languages =
            ResourceBundle.getBundle("ch/praeber/humigraf/resources/languages/Languages");
    //Recording Panel
    private JPanel recordingpanel;
    private Border recordingborder;
    private JLabel IntervallLabel;
    private JComboBox IntervallCombo;
    private JLabel lblpaperfeed;
    private JComboBox cbpaperfeed;
    private JLabel lblsiunit;
    private JComboBox cbsiunit;
    //Alarm Panel
    private JPanel alarmpanel;
    private Border alarmborder;
    private JLabel lblalarm;
    private JSpinner spnalarm;
    private SpinnerListModel modelalarm;
    private JLabel lblminhum;
    private JSpinner spminhum;
    private SpinnerNumberModel modelminhum;
    private JLabel lblmaxhum;
    private SpinnerNumberModel modelmaxhum;
    private JSpinner spmaxhum;
    private JLabel lblmintemp;
    private SpinnerNumberModel modelmintemp;
    private JSpinner spmintemp;
    private JLabel lblmaxtemp;
    private SpinnerNumberModel modelmaxtemp;
    private JSpinner spmaxtemp;
    //Statepanel
    private JPanel statepanel;
    private Border stateborder;
    private JLabel lblstate;
    private JLabel lblbattery;
    private JLabel lblScale;
    private JLabel battery;
    private JLabel scale;
    private int bat = 2;
    private int sc = 0;
    //Buttons
    private JButton CancelButton;
    private JButton SaveButton;
    private JPanel ButtonPanel;
    //Controller
    private Controller control;
    private final String[] alarmvalues = {languages.getString("OFF"), "1", "2", "3", "4", "5",
        "6", "7", "8", "9", languages.getString("ENDLESS (MANUAL STOP)")};
    //settings
    private long activeintervall;
    private int paperfeed;
    private int siunit;
    private int alarm;
    private int minhum;
    private int maxhum;
    private int mintemp;
    private int maxtemp;
    //changed settings
    private long comboint;
    private int newpaperfeed;
    private int newsiunit;
    private int newalarm;
    private int newminhum;
    private int newmaxhum;
    private int newmintemp;
    private int newmaxtemp;
    private boolean intervalchange = false;
    //tooltips
    private String tipinterval;
    private String tippaperfeed;
    private String tipunit;
    private String tipalarm;
    private String tiplowerhum;
    private String tipupperhum;
    private String tiplowertemp;
    private String tipuppertemp;

    /**
     * Start the Settings Dialog. Allows to configure Humigraf
     *
     * @param mainFrame Frame which opens a Configure Dialog
     * @param control Controller used to configure Humigraf
     */
    public DeviceSettingsDialog(AnalyticsGUI parent, Controller control) {
        super(parent);
        this.control = control;
        setTitle(languages.getString("HUMIGRAF SETTINGS"));
        setModal(true);
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
        //setSize(420, 400);
        initToolTipStrings();

        setLayout(new BorderLayout());
        loadConfigFromHumigraf();
        initialise();
        setupListeners();
        pack();
        setLocationRelativeTo(parent);
        setMinimumSize(getSize());
        //  setResizable(false);

        setVisible(true);
    }

    /**
     * initialies ToolTipStrings
     */
    private void initToolTipStrings() {
        tipinterval = languages.getString("INTERVAL BETWEEN TWO RECORDS. CHANGING THIS VALUE WILL ERASE ALL RECORDS FROM HUMIGRAF.");
        tippaperfeed = languages.getString("CONFIGURE THE PAPER FEED SPEED.");
        tipunit = languages.getString("ADJUST THE PHYSICAL UNITS WHICH ARE USED ON PAPER AND DISPLAY.");
        tipalarm = languages.getString("ADJUST THE NUMBER OF SOUNDS PLAYED WHEN ONE OF THE ALARM LEVELS HAS BEEN CROSSED.");
        tiplowerhum = languages.getString("CONFIGURE THE LOWER ALARM LEVEL FOR HUMIDITY.");
        tipupperhum = languages.getString("CONFIGURE THE UPPER ALARM LEVEL FOR HUMIDITY.");
        tiplowertemp = languages.getString("CONFIGURE THE LOWER ALARM LEVEL FOR TEMPERATURE.");
        tipuppertemp = languages.getString("CONFIGURE THE UPPER ALARM LEVEL FOR TEMPERATURE.");
    }

    /**
     * Initialises Buttons, Panels and constructs the Window
     */
    private void initialise() {
        //Greate Button Space
        CancelButton = new JButton(languages.getString("CANCEL"));
        SaveButton = new JButton(languages.getString("SAVE"));
        ButtonPanel = new JPanel();
        ButtonPanel.setLayout(new FlowLayout(FlowLayout.TRAILING));
        ButtonPanel.add(SaveButton);
        ButtonPanel.add(CancelButton);


        //Recording Panel
        recordingpanel = new JPanel();
        recordingborder = BorderFactory.createTitledBorder(languages.getString("RECORDING"));
        recordingpanel.setBorder(recordingborder);
        recordingpanel.setLayout(new GridLayout(3, 2));
        lblpaperfeed = new JLabel(languages.getString("<HTML>PAPER FEED</HTML>"));
        cbpaperfeed = new JComboBox();
        cbpaperfeed.setToolTipText(tippaperfeed);
        cbpaperfeed.addItem("0.5 mm / Min");
        cbpaperfeed.addItem("1.0 mm / Min");
        cbpaperfeed.addItem("2.0 mm / Min");
        cbpaperfeed.addItem("4.0 mm / Min");
        cbpaperfeed.setSelectedIndex(paperfeed);

        lblsiunit = new JLabel(languages.getString("<HTML>SI-UNIT ON HUMIGRAF</HTML>"));
        cbsiunit = new JComboBox();
        cbsiunit.setToolTipText(tipunit);
        cbsiunit.addItem(languages.getString("FAHRENHEIT"));
        cbsiunit.addItem(languages.getString("CELSIUS"));
        cbsiunit.setSelectedIndex(siunit);

        IntervallLabel = new JLabel(languages.getString("<HTML>RECORDING TIME INTERVALL</HTML>"));
        IntervallCombo = new JComboBox();
        IntervallCombo.setToolTipText(tipinterval);
        IntervallCombo.setModel(new javax.swing.DefaultComboBoxModel());
        IntervallCombo.addItem(languages.getString("OFF"));
        IntervallCombo.addItem("1 Min (45 " + languages.getString("DAYS") + ")");
        IntervallCombo.addItem("2 Min (91 " + languages.getString("DAYS") + ")");
        IntervallCombo.addItem("4 Min (182 " + languages.getString("DAYS") + ")");
        IntervallCombo.addItem("6 Min (274 " + languages.getString("DAYS") + ")");
        IntervallCombo.addItem("8 Min (365 " + languages.getString("DAYS") + ")");
        //IntervallCombo.addItem("60 Min (2738 " + languages.getString("DAYS") + ")");
        IntervallCombo.setEditable(false);
        IntervallCombo.setSelectedIndex((int) activeintervall);

        recordingpanel.add(IntervallLabel);
        recordingpanel.add(IntervallCombo);
        //recordingpanel.add(lblpaperfeed);
        //recordingpanel.add(cbpaperfeed);
        recordingpanel.add(lblsiunit);
        recordingpanel.add(cbsiunit);

        //Alarm panel
        alarmpanel = new JPanel();
        alarmborder = BorderFactory.createTitledBorder(languages.getString("ALARM"));
        alarmpanel.setBorder(alarmborder);
        alarmpanel.setLayout(new GridLayout(6, 2));

        lblalarm = new JLabel(languages.getString("ALARM (NUMBER OF BEEPS)"));
        modelalarm = new SpinnerListModel(alarmvalues);
        modelalarm.setValue(alarmvalues[alarm]);
        spnalarm = new JSpinner(modelalarm);
        spnalarm.setToolTipText(tipalarm);
        alarmpanel.add(lblalarm);
        alarmpanel.add(spnalarm);

        modelminhum = new SpinnerNumberModel(minhum, 0, 100, 1);
        lblminhum = new JLabel(languages.getString("LOWER HUMIDITY LIMIT"));
        spminhum = new JSpinner(modelminhum);
        spminhum.setToolTipText(tiplowerhum);
        alarmpanel.add(lblminhum);
        alarmpanel.add(spminhum);

        lblmaxhum = new JLabel(languages.getString("UPPER HUMIDITY LIMIT"));
        modelmaxhum = new SpinnerNumberModel(maxhum, 0, 100, 1);
        spmaxhum = new JSpinner(modelmaxhum);
        spmaxhum.setToolTipText(tipupperhum);
        alarmpanel.add(lblmaxhum);
        alarmpanel.add(spmaxhum);


        modelmintemp = new SpinnerNumberModel(mintemp, -5, 45, 1);
        lblmintemp = new JLabel(languages.getString("LOWER TEMPERATURE LIMIT"));
        spmintemp = new JSpinner(modelmintemp);
        spmintemp.setToolTipText(tiplowertemp);
        alarmpanel.add(lblmintemp);
        alarmpanel.add(spmintemp);

        lblmaxtemp = new JLabel(languages.getString("UPPER TEMPERATURE LIMIT"));
        modelmaxtemp = new SpinnerNumberModel(maxtemp, -5, 45, 1);
        spmaxtemp = new JSpinner(modelmaxtemp);
        spmaxtemp.setToolTipText(tipuppertemp);
        alarmpanel.add(lblmaxtemp);
        alarmpanel.add(spmaxtemp);

        //statepanel
        statepanel = new JPanel();
        stateborder = BorderFactory.createTitledBorder(languages.getString("STATE"));
        statepanel.setBorder(stateborder);
        statepanel.setLayout(new GridLayout(2, 2));

        lblbattery = new JLabel(languages.getString("BATTERY_STATE"));
        if (bat == 2) {
            battery = new JLabel(languages.getString("GOOD"));
        } else if (bat == 1) {
            battery = new JLabel(languages.getString("LOW"));
        } else {
            battery = new JLabel(languages.getString("EMPTY"));
        }


        lblScale = new JLabel(languages.getString("PAPER_SCALE"));        
        if (sc == 0) {
            scale = new JLabel(languages.getString("STANDARD") + Fahrenheit.getSIUnit());
        } else {
            scale = new JLabel(languages.getString("REDUCED") + Fahrenheit.getSIUnit());
        }


        statepanel.add(lblbattery);
        statepanel.add(battery);
        statepanel.add(lblScale);
        statepanel.add(scale);

        //Add the Components to the Window
        JPanel middlepanel = new JPanel();
        middlepanel.setLayout(new BorderLayout());
        middlepanel.add(alarmpanel, BorderLayout.CENTER);
        middlepanel.add(statepanel, BorderLayout.SOUTH);

        add(recordingpanel, BorderLayout.NORTH);
        add(middlepanel, BorderLayout.CENTER);
        add(ButtonPanel, BorderLayout.SOUTH);
    }

    /**
     * Setup all listener
     */
    private void setupListeners() {
        //Cancel_Button Listener
        CancelButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                cancelAction();
            }
        });

        SaveButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                saveAction();
            }
        });
        //Comboboxes
        IntervallCombo.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                comboint = IntervallCombo.getSelectedIndex();
                intervalchange = true;
            }
        });

        cbsiunit.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                newsiunit = cbsiunit.getSelectedIndex();
            }
        });

        cbpaperfeed.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                newpaperfeed = cbpaperfeed.getSelectedIndex();
            }
        });


        //alarm event
        spnalarm.addChangeListener(new ChangeListener() {

            @Override
            public void stateChanged(ChangeEvent e) {
                String a = (String) modelalarm.getValue();
                if (a.equals(alarmvalues[0])) {
                    newalarm = 0;
                } else if (a.equals(alarmvalues[10])) {
                    newalarm = 10;
                } else {
                    newalarm = Integer.parseInt(a);
                }
            }
        });

        //Spinner Events
        spminhum.addChangeListener(new ChangeListener() {

            @Override
            public void stateChanged(ChangeEvent e) {
                newminhum = modelminhum.getNumber().intValue();
            }
        });


        spmaxhum.addChangeListener(new ChangeListener() {

            @Override
            public void stateChanged(ChangeEvent e) {
                newmaxhum = modelmaxhum.getNumber().intValue();
            }
        });

        spmintemp.addChangeListener(new ChangeListener() {

            @Override
            public void stateChanged(ChangeEvent e) {
                newmintemp = modelmintemp.getNumber().intValue();
            }
        });


        spmaxtemp.addChangeListener(new ChangeListener() {

            @Override
            public void stateChanged(ChangeEvent e) {
                newmaxtemp = modelmaxtemp.getNumber().intValue();
            }
        });


    }

    /**
     * Loads the configuration from Humigraf.
     */
    private void loadConfigFromHumigraf() {
        //Load Interval
        activeintervall = control.getIntervall();
        switch ((int) activeintervall) {
            case 0:
                activeintervall = (long) 0;
                break;
            case 1:
                activeintervall = 1;
                break;
            case 2:
                activeintervall = (long) 2;
                break;
            case 4:
                activeintervall = (long) 3;
                break;
            case 6:
                activeintervall = (long) 4;
                break;
            case 8:
                activeintervall = (long) 5;
                break;
            default:
                activeintervall = (long) -1;
        }

//        //load paperfeed
//        paperfeed = control.getPaperFeed();
//        if (paperfeed < 0) {
//            paperfeed = 0;
//        }
        newpaperfeed = paperfeed;
        siunit = control.getTemperatureUnit();
        if (siunit < 0) {
            siunit = 0;
        }
        newsiunit = siunit;

        alarm = control.getAlarm();
        if (alarm < 0 || alarm > 10) {
            alarm = 0;
        }
        newalarm = alarm;

        minhum = control.getMinHum();
        if (minhum < 0) {
            minhum = 0;
        }
        newminhum = minhum;

        maxhum = control.getMaxHum();
        if (maxhum < 0) {
            maxhum = 0;
        }
        newmaxhum = maxhum;

        mintemp = control.getMinTemp();
        if (mintemp < -5) {
            mintemp = 0;
        }
        newmintemp = mintemp;

        maxtemp = control.getMaxTemp();
        if (maxtemp < -5) {
            maxtemp = 0;
        }
        newmaxtemp = maxtemp;

         sc = control.getDeviceSector();
        bat = control.getBatteryState();
    }

    /**
     * Clear all actions done.
     */
    private void cancelAction() {
        //Clear all necessery data from the devices etc
        setVisible(false);
        this.removeAll();

    }

    /**
     * Action if savebutton is pressed.
     */
    private void saveAction() {
        if (writeToHumigraf()) {
            setVisible(false);
            this.removeAll();
        }
    }

    /**
     * Writes the changed elements to the humigraf
     */
    private boolean writeToHumigraf() {
        boolean answ = true;
        if (comboint != activeintervall && intervalchange) {
            int response = 0;
            int value = (int) comboint;
            response = JOptionPane.showConfirmDialog(this,
                    languages.getString("SETTING THE RECORDING INTERVAL WILL REMOVE ALL RECORDS FROM")
                    + languages.getString("HUMIGRAF. WOULD YOU LIKE TO CONTINUE AND REMOVE ALL RECORDS")
                    + languages.getString("FROM HUMIGRAF BY SETTING THE INTERVAL?"),
                    languages.getString("CAUTION"), JOptionPane.YES_NO_OPTION,
                    JOptionPane.WARNING_MESSAGE);
            // Response value: Yes = 0, No = 1, Cancel = 2
            if (response == JOptionPane.YES_OPTION) {
                control.setInterval(value);
            } else {
                answ = false;
                //Maybe reset
            }
        }

//        if (newpaperfeed != paperfeed) {
//            control.setPaperFeed(newpaperfeed);
//        }

        if (newsiunit != siunit) {
            control.setTemperatureUnit(newsiunit);
        }

        if (newminhum != minhum) {
            control.setMinHum(newminhum);
        }

        if (newmaxhum != maxhum) {
            control.setMaxHum(newmaxhum);
        }

        if (newmintemp != mintemp) {
            control.setMinTemp(newmintemp);
        }

        if (newmaxtemp != maxtemp) {
            control.setMaxTemp(newmaxtemp);
        }

        if (newalarm != alarm) {
            control.setAlarm(newalarm);
        }
        return answ;
    }
}
