/*
 * Copyright (C) Philipp Meier 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.praeber.humigraf;

import ch.praeber.humigraf.system.Fahrenheit;
import java.awt.BasicStroke;
import java.awt.Color;
import java.text.SimpleDateFormat;
import java.util.ResourceBundle;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.Range;
import org.jfree.data.xy.XYDataset;
import org.jfree.ui.RectangleInsets;

/**
 * HumigrafChartFactory
 * Chart displayed by main window
 * @author Meier Philipp <philipp.meier@stud.hslu.ch>
 */
public class HumigrafChartFactory {

    private static final Range HUMIDITYRANGE = new Range(0.0, 100.0);
    private static final Range TEMPERATURERANGE = new Range(-5.0, 45.0);
    /** Line style: line */
    public static final String STYLE_LINE = "line";
    /** Line style: dashed */
    public static final String STYLE_DASH = "dash";
    /** Line styleDataEntry: dotted */
    public static final String STYLE_DOT = "dot";
    private static final ResourceBundle languages =
            ResourceBundle.getBundle("ch/praeber/humigraf/resources/languages/Languages");

    public static Range getHUMIDITYRANGE() {
        return HUMIDITYRANGE;
    }

    public static Range getTEMPERATURERANGE() {
        return TEMPERATURERANGE;
    }

    public HumigrafChartFactory() {
    }

    public static JFreeChart createChart(XYDataset temperature, XYDataset humidity) {
        JFreeChart chart = ChartFactory.createTimeSeriesChart(
                null, // title
                languages.getString("DATE"),
                // x-axis label
                Fahrenheit.addSIUnit(languages.getString("TEMPERATURE")),
                // y-axis label
                temperature,
                // data
                true,
                // create legend?
                true,
                // generate tooltips?
                false // generate URLs?
                );
        
        chart.setBackgroundPaint(Color.white);
        chart.setAntiAlias(true);
        chart.setTextAntiAlias(true);
        XYPlot plot = (XYPlot) chart.getPlot();
        NumberAxis axis2 = new NumberAxis(languages.getString("HUMIDITY %"));
        axis2.setAutoRange(false);
        axis2.setRange(HUMIDITYRANGE);
        axis2.setAutoRangeIncludesZero(true);
        axis2.setLabelPaint(Color.BLUE);
        axis2.setLabelFont(plot.getRangeAxis(0).getLabelFont());        
    
        plot.getRangeAxis(0).setLabelPaint(Color.RED);
        plot.getRangeAxis(0).setRange(TEMPERATURERANGE);
        plot.getRangeAxis(0).setAutoRange(false);

        //not perfect as it does not display data as whished
        plot.setRangeAxis(1, axis2);
        plot.setDataset(1, humidity);
        plot.mapDatasetToRangeAxis(1, 1);

        //Humidity Renderer
        XYItemRenderer renderer2 = new XYLineAndShapeRenderer(); // up to y
        renderer2.setSeriesStroke(2, toStroke(STYLE_DASH));
        renderer2.setSeriesStroke(1, toStroke(STYLE_DASH));
        renderer2.setSeriesPaint(0, Color.BLUE);
        renderer2.setSeriesPaint(1, Color.BLUE);
        renderer2.setSeriesPaint(2, Color.BLUE);
        renderer2.setBaseItemLabelsVisible(true);
        renderer2.setSeriesShape(0, null);
        renderer2.setSeriesShape(1, null);
        renderer2.setSeriesShape(2, null);
        renderer2.setBaseToolTipGenerator(new ToolTipGenerator(languages.getString("HUMIDITY")));
        plot.setRenderer(1, renderer2);
        
        XYLineAndShapeRenderer renderer3 = (XYLineAndShapeRenderer) plot.getRenderer(1);
        renderer3.setSeriesShapesVisible(0, false);
        renderer3.setSeriesShapesVisible(1, false);
        renderer3.setSeriesShapesVisible(2, false);
        plot.setBackgroundPaint(Color.white);
        plot.setDomainGridlinePaint(Color.lightGray);
        plot.setRangeGridlinePaint(Color.lightGray);
        plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
        plot.setDomainCrosshairVisible(true);
        plot.setRangeCrosshairVisible(true);

        XYItemRenderer r = plot.getRenderer();
        if (r instanceof XYLineAndShapeRenderer) {
            XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) r;
            renderer.setBaseShapesVisible(true);
            renderer.setBaseShapesFilled(true);
            //Set Temperature to Red
            renderer.setSeriesPaint(0, Color.RED);
            renderer.setSeriesPaint(1, Color.RED);
            renderer.setSeriesPaint(2, Color.RED);
            renderer.setSeriesPaint(3, Color.GREEN);
            renderer.setSeriesShapesVisible(0, false);
            renderer.setSeriesShapesVisible(1, false);
            renderer.setSeriesShapesVisible(2, false);
            renderer.setSeriesShapesVisible(3, false);
            //Set Min/Max to Dashes
            renderer.setSeriesStroke(1, toStroke(STYLE_DASH));
            renderer.setSeriesStroke(2, toStroke(STYLE_DASH));
            //renderer.setSeriesStroke(3, toStroke(STYLE_DASH));
        }
        r.setBaseToolTipGenerator(new ToolTipGenerator(
                languages.getString("TEMPERATURE"))); 
        
         NumberAxis axis3 = new NumberAxis
                 (Fahrenheit.addSIUnit(languages.getString("DEW POINT")));
        axis3.setAutoRange(true);        
        axis3.setLabelPaint(Color.GREEN);
        axis3.setAxisLineVisible(false);
        axis3.setMinorTickMarksVisible(false);
        axis3.setTickMarksVisible(false);
        axis3.setTickLabelsVisible(false);
        plot.setRangeAxis(3, axis3);
        plot.setRangeAxisLocation(3, AxisLocation.BOTTOM_OR_LEFT);        
        
        DateAxis axis = (DateAxis) plot.getDomainAxis();
        axis.setDateFormatOverride(new SimpleDateFormat());        
        return chart;
    }

    /**
     * Convert style string to stroke object.
     *
     * @param style One of STYLE_xxx.
     * @return Stroke for <i>style</i> or null if style not supported.
     */
    private static BasicStroke toStroke(String style) {
        BasicStroke result = null;

        if (style != null) {
            float lineWidth = 0.2f;
            float dash[] = {5.0f};
            float dot[] = {lineWidth};

            if (style.equalsIgnoreCase(STYLE_LINE)) {
                result = new BasicStroke(lineWidth);
            } else if (style.equalsIgnoreCase(STYLE_DASH)) {
                result = new BasicStroke(lineWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 10.0f, dash, 0.0f);
            } else if (style.equalsIgnoreCase(STYLE_DOT)) {
                result = new BasicStroke(lineWidth, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, 2.0f, dot, 0.0f);
            }
        }//else: input unavailable

        return result;
    }//toStroke()
}
