/*
 * Copyright (C) Philipp Meier 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.praeber.humigraf.model;

import java.io.Serializable;
import java.util.AbstractCollection;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

/**
 * DataList
 * List of DataEntries, main storage of data
 * @author Meier Philipp
 */
public class DataList extends AbstractCollection<DataEntry> implements Serializable {

    private LinkedList<DataEntry> list = new LinkedList<DataEntry>();
    private double mintemp = 0;
    private double maxtemp = 0;
    private double minhum = 0;
    private double maxhum = 0;
    private boolean level = false;

    /**
     * Initiate a new DataList with certain values for lower /upper alarm levels
     * @param mintemp
     * @param maxtemp
     * @param minhum
     * @param maxhum
     */
    public DataList(double mintemp, double maxtemp, double minhum, double maxhum) {
        this.mintemp = mintemp;
        this.maxtemp = maxtemp;
        this.minhum = minhum;
        this.maxhum = maxhum;
        level = true;
        
    }

    public DataList() {
        
    }

    /**
     * Configure a Min / Max value to see a line displayed in on the GUI where the Entities did cross those lines
     * @param mintemp Min Temperature
     * @param maxtemp Max Temperature
     * @param minhum Min Humidity
     * @param maxhum Max Humidity
     */
    public synchronized void setMinMax(double mintemp, double maxtemp, double minhum, double maxhum) {
        this.mintemp = mintemp;
        this.maxtemp = maxtemp;
        this.minhum = minhum;
        this.maxhum = maxhum;
        level = true;
    }

    /**
     * Return the
     * @return
     */
    public boolean hasMinOrMax() {
        return level;
    }

    /**
     * Add an new Entity to the List
     * @param temp Temperature of the Entity
     * @param hum Humidity of the Entity
     * @param date Date when Data was captured
     */
    public synchronized void addNewEntry(double temp, double hum, long date) {
        list.addFirst(new DataEntry(temp, hum, date));
    }

    /**
     * Add an new Entity to the List
     * @param entry DataEntry entity
     */
    public synchronized void addNewEntry(DataEntry entry) {
        list.addFirst(entry);
    }

    /**
     * Add a new Live Captured Entity to the List
     * @param temp Temperature of the Entity
     * @param hum Humidity of the Entity
     * @param date Date when Data was captured
     */
    public synchronized void addLiveEntry(double temp, double hum, long date) {
        list.add(new DataEntry(temp, hum, date));
    }

    public synchronized void addLiveEntry(DataEntry entry) {
        list.add(entry);
    }

    /**
     *
     * @return Min Value of Temperature, this Line should not be crossed
     */
    public synchronized double getMinTemp() {
        return mintemp;
    }

    /**
     *
     * @return Max Value of Temperature, this Line should not be crossed
     */
    public synchronized double getMaxTemp() {
        return maxtemp;
    }

    /**
     *
     * @return Min Value of Humidity, this Line should not be crossed
     */
    public synchronized double getMinHum() {
        return minhum;
    }

    /**
     *
     * @return Max Value of Humidity, this Line should not be crossed
     */
    public synchronized double getMaxHum() {
        return maxhum;
    }

    /**
     *
     * @return Date when first entity was captured
     */
    public synchronized Date getFirstTime() {
        return list.getFirst().getDate();
    }

    /**
     *
     * @return Date when last entity was captured
     */
    public synchronized Date getLastTime() {
        return list.getLast().getDate();
    }

    /**
     *
     * @return true if list is empty
     */
    @Override
    public synchronized boolean isEmpty() {
        return list.isEmpty();
    }

    /**
     * Returns list iterator
     * @return
     */
    @Override
    public Iterator iterator() {
        return list.iterator();
    }

    /**
     * Returns size of list
     * @return interger
     */
    @Override
    public int size() {
        return list.size();
    }
}
