/*
 * Copyright (C) Philipp Meier 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.praeber.humigraf.model;

import ch.praeber.humigraf.system.Fahrenheit;
import ch.praeber.humigraf.system.ThreadErrors;
import java.io.Serializable;
import java.util.Locale;
import java.util.Observable;
import java.util.ResourceBundle;
import org.jfree.data.time.Minute;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;

/**
 * DataManager
 * GUI Data, this Class contains the Data displayed on the gui.
 * @author Meier Philipp <philipp.meier@stud.hslu.ch>
 */
//TODO better Threadsafety
public class DataManager extends Observable implements Serializable {

    private static final ResourceBundle languages = ResourceBundle.getBundle("ch/praeber/humigraf/resources/languages/Languages");
    private TimeSeriesCollection ChartDataTemp;
    private TimeSeriesCollection ChartDataHum;
    private DataList datalist;
    //private boolean dataLoaded = false;
    //private boolean datasaved = true;
    private ThreadErrors threaderror = ThreadErrors.NOERROR;

    public DataManager() {
        initialise();
    }

    /**
     * Initialises all necessary Datastructurs
     */
    private void initialise() {
        ChartDataTemp = initialiseTemperatureDataset();
        ChartDataHum = initialiseHumidityDataset();
        datalist = new DataList();
    }

    /**
     * Initialises the Temperature Dataset for the Chart
     * @return XYDataset used by the Chart
     */
    private TimeSeriesCollection initialiseTemperatureDataset() {
        TimeSeries s1 = new TimeSeries(languages.getString("TEMPERATURE"));
        TimeSeries s2 = new TimeSeries(languages.getString("MINIMAL TEMPERATURE"));
        TimeSeries s3 = new TimeSeries(languages.getString("MAXIMAL TEMPERATURE"));
        TimeSeries s4 = new TimeSeries(languages.getString("DEW POINT"));

        TimeSeriesCollection dataset = new TimeSeriesCollection();
        dataset.addSeries(s1);
        dataset.addSeries(s2);
        dataset.addSeries(s3);
        dataset.addSeries(s4);
        //    dataset.setDomainIsPointsInTime(true);
        return dataset;
    }

    /**
     * Initialises the Temperature Dataset for the Chart
     * @return XYDataset used by the Chart
     */
    private TimeSeriesCollection initialiseHumidityDataset() {
        TimeSeries s1 = new TimeSeries(languages.getString("HUMIDITY"));
        TimeSeries s2 = new TimeSeries(languages.getString("MINIMAL HUMIDITY"));
        TimeSeries s3 = new TimeSeries(languages.getString("MAXIMAL HUMIDITY"));

        TimeSeriesCollection dataset = new TimeSeriesCollection();
        dataset.addSeries(s1);
        dataset.addSeries(s2);
        dataset.addSeries(s3);
        //  dataset.setDomainIsPointsInTime(true);
        return dataset;
    }

    /**
     * Adds a Value captured at a certain Minute to the graphical Chart
     *
     * @param latest Minute when the Caputure took place
     * @param value Value that needs to be added to the GUI Chart
     */
    private void addTempChart(DataEntry entry) {
        double value = entry.getTemp();
        double dew = entry.getTau();
        Locale locale = Locale.getDefault();
        if (locale.equals(Locale.US)) {
            value = Fahrenheit.toFahrenheit(value);
            dew = Fahrenheit.toFahrenheit(dew);
        }
        ChartDataTemp.getSeries(0).add(new Minute(entry.getDate()), value, false);
        ChartDataTemp.getSeries(3).add(new Minute(entry.getDate()), dew, false);
    }

    /**
     * Adds a certain Value to be set as Min Value to the Temp Chart
     * @param value Value to be set
     */
    private void addTempMinChart(Double value) {
        Locale locale = Locale.getDefault();
        if (locale.equals(Locale.US)) {
            value = Fahrenheit.toFahrenheit(value);
        }
        ChartDataTemp.getSeries(1).add(new Minute(datalist.getFirstTime()), value);
        ChartDataTemp.getSeries(1).add(new Minute(datalist.getLastTime()), value);
    }

    /**
     * Adds a certain Value to be set as Max Value to the Temp Chart
     * @param value Value to be set
     */
    private void addTempMaxChart(Double value) {
        Locale locale = Locale.getDefault();
        if (locale.equals(Locale.US)) {
            value = Fahrenheit.toFahrenheit(value);
        }
        ChartDataTemp.getSeries(2).add(new Minute(datalist.getFirstTime()), value);
        ChartDataTemp.getSeries(2).add(new Minute(datalist.getLastTime()), value);
    }

    /**
     * Adds a Value captured at a certain Minute to the graphical Chart
     *
     * @param latest Minute when the Caputure took place
     * @param value Value that needs to be added to the GUI Chart
     */
    private void addHumChart(DataEntry entry) {
        ChartDataHum.getSeries(0).add(new Minute(entry.getDate()),
                entry.getHum(), false);
    }

    /**
     * Adds a certain Value to be set as Min Value to the Hum Chart
     * @param value Value to be set
     */
    private void addHumMinChart(Double value) {
        ChartDataHum.getSeries(1).add(new Minute(datalist.getFirstTime()), value);
        ChartDataHum.getSeries(1).add(new Minute(datalist.getLastTime()), value);
    }

    /**
     * Adds a certain Value to be set as Min Value to the Hum Chart
     * @param value Value to be set
     */
    private void addHumMaxChart(Double value) {
        ChartDataHum.getSeries(2).add(new Minute(datalist.getFirstTime()), value);
        ChartDataHum.getSeries(2).add(new Minute(datalist.getLastTime()), value);

    }

    /**
     *
     * @return YXDataset which is used for the chart (Temperature)
     */
    public XYDataset getChartTemp() {
        return ChartDataTemp;

    }

    /**
     *
     * @return YXDataset which is used for the chart (Humidity)
     */
    public XYDataset getChartHum() {
        return ChartDataHum;
    }

    /**
     * Checks if data is empty
     * @return true if empty
     */
    public boolean isEmpty() {
        return datalist.isEmpty();
    }

    /**
     * Add an new Entity to the List
     * @param entry add new DataEntry to the ui
     */
    public void addNewEntry(DataEntry entry) {
        datalist.addNewEntry(entry);
        addHumChart(entry);
        addTempChart(entry);
    }

    /**
     * Add an new Entity to the List, from File
     * @param entry
     */
    public void addEntryfromFile(DataEntry entry) {
        datalist.addLiveEntry(entry); //Because it follows the correct order
        addHumChart(entry);
        addTempChart(entry);
    }

    /**
     * Add a new Live Captured Entity to the List
     * @param entry add new DataEntry to the ui
     */
    public void addLiveEntry(DataEntry entry) {
        datalist.addLiveEntry(entry);
        addHumChart(entry);
        addTempChart(entry);
        //setDataFromFile(false);
        updateGUI();
    }

    /**
     *
     * @param mintemp
     * @param maxtemp
     * @param minhum
     * @param maxhum
     */
    public void setMinMax(double mintemp, double maxtemp, double minhum, double maxhum) {
        datalist.setMinMax(mintemp, maxtemp, minhum, maxhum);
        addHumMaxChart(datalist.getMaxHum());
        addHumMinChart(datalist.getMinHum());
        addTempMaxChart(datalist.getMaxTemp());
        addTempMinChart(datalist.getMinTemp());
    }
    
    //Needs to be removed

    public DataList getList() {
        return datalist;

    }

    /**
     * Returns the Size of the Datalist
     * @return Size of Datalist
     */
    public synchronized int getSize() {
        return datalist.size();
    }

//    /**
//     * Adds an Entry to GUI
//     *
//     * @param number Number of DataEntry in datalist that will be added
//     */
//    public boolean isDataSaved() {
//        return datasaved;
//    }
//
//    /**
//     * Set if data has been saved in a pra file
//     * @param DataIsSaved
//     */
//    public void setDataSaved(boolean DataIsSaved) {
//        this.datasaved = DataIsSaved;
//    }

//    public boolean isDataFromFile() {
//        return dataLoaded;
//    }
//
//    /**
//     * Sets true if data has been loaded from device
//     * @param dataLoaded
//     */
//    public void setDataFromFile(boolean dataLoaded) {
//        this.dataLoaded = dataLoaded;
//        setDataSaved(dataLoaded);
//    }

    public void notifyError(ThreadErrors a) {
        threaderror = a;
        setChanged();
        notifyObservers();
    }

    /**
     * Lets the UI get the Errorit recieved.
     * @return ThreadErrors
     */
    public ThreadErrors getError() {
        ThreadErrors temp = this.threaderror;
        this.threaderror = ThreadErrors.NOERROR;
        return temp;
    }

    public void updateGUI() {
//        if (isDataFromFile()) {
//            setDataSaved(true);
//        } else {
//            setDataSaved(false);
//        }
        setChanged();
        notifyObservers();
        notifySeriesOfUpdate();
    }

    private synchronized void notifySeriesOfUpdate() {
        ChartDataHum.getSeries(0).fireSeriesChanged();
        ChartDataTemp.getSeries(0).fireSeriesChanged();
    }

    public synchronized boolean clearData() {
        boolean result = false;
        //setDataFromFile(false);
//        setDataSaved(true);
        //Clear List
        datalist.clear();
        //Clear Graph
        ChartDataHum.getSeries(0).clear();
        ChartDataHum.getSeries(1).clear();
        ChartDataHum.getSeries(2).clear();
        ChartDataTemp.getSeries(0).clear();
        ChartDataTemp.getSeries(1).clear();
        ChartDataTemp.getSeries(2).clear();
        ChartDataTemp.getSeries(3).clear();
        return result;
    }
}
