/*
 * Copyright (C) Philipp Meier 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.praeber.humigraf.system;

import ch.praeber.humigraf.model.DataEntry;
import ch.praeber.humigraf.model.DataList;
import ch.praeber.humigraf.model.DataManager;
import java.io.File;
import java.util.Date;
import javax.swing.ProgressMonitor;

/**
 * Controller
 *
 * @author Meier Philipp <philipp.meier@stud.hslu.ch>
 */
public class Controller {
    //TODO better threadsafety

    private boolean threadactive = false;
    private Thread worker;
    private DataManager datamanager;
    private long millis = 0;
    private final static boolean DEBUGMODE = true;
    private iControl humigraf;
    private boolean saved = true;

    public boolean isSaved() {
        return saved;
    }

    public void setSaved(boolean saved) {
        this.saved = saved;
    }
    
    public final boolean isDebugmode() {
        return DEBUGMODE;
    }

    public Controller(DataManager Manager) {
        datamanager = Manager;
        if (isDebugmode()) {
            humigraf = new HumigrafSimulator();
        } else {
            humigraf = new DeviceControl();
        }
    }

    public synchronized boolean isThreadActive() {
        return threadactive;
    }

    public synchronized void stopThread() {
        threadactive = false;
    }

    public synchronized void startThread() {
        threadactive = true;
        worker.start();
    }

    public synchronized int getInterval() {
        return (int) (millis / 60000);
    }

// <editor-fold defaultstate="collapsed" desc="File Operations">
    /**
     * Controller Method to load File into Memory.
     *
     * @param filename
     * @return 0 on Success -1 && -2 for Errors
     */
    public synchronized int loadFile(String filename) {
        //Is Broken use normal methods
        if (filename.equals("-1")) {
            return 1;
        } else {
            Object[] dataObjects = FileHandling.readBinFile(filename);
            if (dataObjects == null) {
                return -2;
            } else {
                DataList temp = (DataList) dataObjects[0];
                for (DataEntry a : temp) {
                    datamanager.addEntryfromFile(a);
                }
                if (temp.hasMinOrMax()) {
                    datamanager.setMinMax(temp.getMinTemp(), temp.getMaxTemp(),
                            temp.getMinHum(), temp.getMaxHum());

                }

                //datamanager.setDataFromFile(true);
                datamanager.updateGUI();
                return 0;
            }
        }
    }

    /**
     *
     * @param file
     * @return
     */
    public synchronized boolean saveToFile(File file) {
        Object[] dataObjects = {datamanager.getList()};
        if (FileHandling.writeBinFile(file, dataObjects)) {
           // datamanager.setDataSaved(true);
            this.updateUiChart();
            return true;
        } else {
            return false;
        }
    }

    /**
     * Export Data through CSVExporter to a CSV formattet File.
     *
     * @param file
     * @return
     */
    public synchronized boolean exportCSV(File file) {
        return new ExportCSV(datamanager).exportCSV(file);
    }

    public synchronized boolean importData(int device, ProgressMonitor dial,
            boolean removedata) {
        if (!isThreadActive()) {
            ImportDataThread thr = new ImportDataThread(device, this,
                    dial, removedata);
            worker = new Thread(thr);
            this.startThread();
            return true;
        } else {
            return false;
        }
    }
    //</editor-fold>

// <editor-fold defaultstate="collapsed" desc="Thread Operations">
    /**
     *
     * @param device
     * @param minute
     * @return
     */
    public synchronized boolean startLiveTracking(int device, int minute) {
        millis = minute * 60000;
        if (!isThreadActive()) {
            worker = new Thread(new LiveTrackingThread(device, millis, this));
            return true;
        } else {
            return false;
        }
    }

    /**
     * Threads send ErrorCodes to the Controller to send Errors.
     *
     * @param i
     */
    void threadError(ThreadErrors i) {
        this.stopThread();
        datamanager.notifyError(i);
    }
// </editor-fold>

    /**
     *
     * @return
     */
    public synchronized boolean resetChart() {
        return datamanager.clearData();
    }

// <editor-fold defaultstate="collapsed" desc="Humigraf Operations">
    /**
     * Loads the Intervall through the getIntervall of an iControl compatible
     * class
     *
     * @return intervall in minutes or -1 if intervall could not be load
     */
    public synchronized long getIntervall() {
        //TODO move callculation to device
        int device = 0;
        long interv = humigraf.getInterval(device);
        System.out.println("Intervall: " + interv);
        return interv;
    }

    /**
     * Minute
     *
     * @param intervall
     * @return
     */
    public synchronized int setInterval(int intervall) {
        return humigraf.setInterval(intervall, 0);
    }

    /**
     *
     * @param device
     * @return
     */
    public synchronized double[] getActualValues(int device) {
        return humigraf.getAcctualValues(device);
    }

//    public synchronized double[] readMemory(int device) {
//        return humigraf.readMemory(device);
//    }
    public synchronized int getMemorySize() {
        return humigraf.getMemorySize();
    }

    public synchronized int requestMemoryDump() {
        return humigraf.requestMemoryDump();
    }

    public synchronized double[] getDataPackage() {
        return humigraf.getDataPackage();
    }

    public synchronized int getPaperFeed() {
        return humigraf.getPaperFeed(0);
    }

    public synchronized int setPaperFeed(int paperfeed) {
        return humigraf.setPaperFeed(0, paperfeed);
    }

    public synchronized int getTemperatureUnit() {
        return humigraf.getTemperatureUnit(0);
    }

    public synchronized int setTemperatureUnit(int siunit) {
        return humigraf.setTemperatureUnit(0, siunit);
    }

    public synchronized int getAlarm() {
        return humigraf.getAlarm(0);
    }

    public synchronized int setAlarm(int alarm) {
        return humigraf.setAlarm(0, alarm);
    }

    public synchronized int getMinHum() {
        return humigraf.getMinHum(0);
    }

    public synchronized int setMinHum(int humidity) {
        return humigraf.setMinHum(0, humidity);
    }

    public synchronized int getMaxHum() {
        return humigraf.getMaxHum(0);
    }

    public synchronized int setMaxHum(int humidity) {
        return humigraf.setMaxHum(0, humidity);
    }

    public synchronized int getMinTemp() {
        return humigraf.getMinTemp(0);
    }

    public synchronized int setMinTemp(int temperature) {
        return humigraf.setMinTemp(0, temperature);
    }

    public synchronized int getMaxTemp() {
        return humigraf.getMaxTemp(0);
    }

    public synchronized int setMaxTemp(int temperature) {
        return humigraf.setMaxTemp(0, temperature);
    }

    public synchronized int abortImport() {
        return humigraf.abort(0);
    }

    public synchronized int formatDevice() {
        return humigraf.deleteMemory(0);
    }

    public synchronized int getBatteryState() {
        return humigraf.getBatteryState(0);
    }

    public synchronized int getDeviceSector() {
        return humigraf.getDeviceSector(0);
    }

    //</editor-fold>
    
// <editor-fold defaultstate="collapsed" desc="DataManager Operations">
    public void addLiveEntry(double temp, double hum, Date date) {
        datamanager.addLiveEntry(new DataEntry(temp, hum, date.getTime()));
    }

    public void addNewEntry(double temp, double hum, Date date) {
        datamanager.addNewEntry(new DataEntry(temp, hum, date.getTime()));
    }

    public void setMinMax() {
        datamanager.setMinMax(humigraf.getMinTemp(0), humigraf.getMaxTemp(0),
                humigraf.getMinHum(0), humigraf.getMaxHum(0));
    }

    public void updateUiChart() {
        datamanager.updateGUI();
    }
//</editor-fold>
}
