/*
 * Copyright (C) Paul Räber 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.praeber.humigraf.system;

import java.util.Date;
import javax.swing.ProgressMonitor;
import javax.swing.SwingWorker;

/**
 * ImportDataThread 
 * Thread to import data from Humigraf.
 * @author Meier Philipp
 */
public class ImportDataThread extends SwingWorker<Boolean, Object> {

    private Controller control;
    private double[] data;
    private int device;
    private long interval;
    private Date date;
    private ProgressMonitor monitor;
    private int memorysize = 0;
    private boolean removedata;

    /**
     *
     * @param device DeviceNumber currently not supported.
     * @param control Controller to work with Humigraf
     * @param monitor ProgressMonitor
     * @param removedata if data should be removed from device after import.
     */
    public ImportDataThread(int device, Controller control,
            ProgressMonitor monitor, boolean removedata) {
        this.control = control;
        this.device = device;
        this.monitor = monitor;
        this.removedata = removedata;
    }

    /**
     * Works as a Background Thread.
     * @return true if successfully.
     * @throws Exception
     */
    @Override
    protected Boolean doInBackground() throws Exception {
        int returnvalue = control.abortImport();
        if (returnvalue == 0) {
            interval = control.getIntervall() * 60000;
            if (interval > 0) {
                //Set current time
                date = new Date();
                memorysize = control.getMemorySize();
                if (memorysize >= 2) {
                    monitor.setMaximum(memorysize);                    
                    /*
                     * This is blocking unfortunately usbhid does currently
                     * not offer any timeout ability.
                     */
                    int answ = control.requestMemoryDump();
                    int counter = 0;
                    if (answ == 0) {
                        int packagecounter = 0;
                        while (packagecounter < memorysize && !monitor.isCanceled()) {
                            double[] result = control.getDataPackage();
                            for (int i = 0; i < result.length; i = i + 2) {
                                control.addNewEntry(result[i], result[i + 1],
                                        new Date(date.getTime()
                                        - ((counter / 2) * interval)));
                                counter = counter + 2;

                                monitor.setProgress(counter);
                                monitor.setNote("Importing: " + counter + " of "
                                        + memorysize);
                            }
                            packagecounter += result.length;
                        }
                        control.setMinMax();
                    } else {
                        monitor.close();
                        control.stopThread();
                        control.resetChart();
                        control.abortImport();
                        control.threadError(ThreadErrors.HUMIGRAFCONNECTIONERROR);
                    }
                }
            } else {
                control.stopThread();
                control.resetChart();
                control.abortImport();
                control.threadError(ThreadErrors.HUMIGRAFINTERVALLERROR);
            }


            if (monitor.isCanceled()) {
                control.resetChart();
                control.stopThread();
                control.abortImport();
            } else {
                if (removedata) {
                    control.formatDevice();
                }
                control.stopThread();
                control.updateUiChart();
            }
            control.setSaved(false);
            return true;
        } else {
            control.stopThread();
            control.resetChart();
            control.abortImport();
            control.threadError(ThreadErrors.HUMIGRAFCONNECTIONERROR);
        }
        return false;
    }
}
