/*
 * Copyright (C) Philipp Meier 2011
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.praeber.humigraf.system;

/**
 * Interface specifying necessary and / or supportet methods of the Humigraf.
 * Version is referring to the supported Humigraf version.
 *
 * @author Philipp Meier
 * @version 3.1
 */
public interface iControl {
    /*
     * CONFIGURATION
     */

    /**
     * Sets the intervall according to the int value Achtung: Schreiben auf
     * diesen Wert initialisiert den Datenspeicher bzw. löscht die aktuelle
     * Aufzeichnung.
     *
     * @param interval 0 = Aufzeichnung ausgeschaltet 1 = 1 Minute 2 = 5 Minuten
     * 3 = 10 Minuten 4 = 15 Minuten 5 = 30 Minuten 6 = 60 Minuten
     * @param device
     * @return 0 on success -1 Error establishing connection with device. -2
     * Error sending command to Humigraf -3 value not supportet
     */
    int setInterval(int interval, int device);

    /**
     * Gets the Interval in coded fromat from the device.
     *
     * @param device Currently not supportet
     * @return integer with one of the following numbers 0 = Recording turned
     * off 1 = 1 Minute 2 = 5 Minutes 3 = 10 Minutes 4 = 15 Minutes 5 = 30
     * Minutes 6 = 60 Minutes -1 Error establishing connection with device. -2
     * Error sending command to Humigraf -3 Error interval not supportet
     */
    int getInterval(int device);

    /**
     * Set the speed of how fast the paper will be moved forward.
     *
     * @param device DeviceNumber, currently not supported.
     * @param paperfeed int value between: 0 = 0.5 mm per min 1 = 1.0 mm per min
     * 2 = 2.0 mm per min 3 = 4.0 mm per min
     * @return 0 on success -1 Error establishing connection with device. -2
     * Error sending command to Humigraf -3 Error paperfeed not supportet
     */
    int setPaperFeed(int device, int paperfeed);

    /**
     * Get the speed of how fast the paper is moved forward.
     *
     * @param device DeviceNumber, currently not supported.
     * @return integer smaller than 0 on failure otherwise 0 = 0.5 mm per min 1
     * = 1.0 mm per min 2 = 2.0 mm per min 3 = 4.0 mm per min -1 Error
     * establishing connection with device. -2 Error sending command to Humigraf
     * -3 Error reading recieved value
     */
    int getPaperFeed(int device);

    /**
     * Sets the SI-Unit which the Humigraf uses on his display / paper.
     *
     * @param device DeviceNumber, currently not supported.
     * @param unit 0 for Celsius, 1 for Fahrenheit
     * @return 0 on success -1 Error establishing connection with device. -2
     * Error sending command to Humigraf -3 Error unit not supportet
     */
    int setTemperatureUnit(int device, int unit);

    /**
     * Returns the SI-Unit which the Humigraf uses on his display / paper.
     *
     * @param device DeviceNumber, currently not supported.
     * @return 0 for Celsius, 1 for Fahrenheit -1 Error establishing connection
     * with device. -2 Error sending command to Humigraf -3 Error reading
     * recieved value
     */
    int getTemperatureUnit(int device);

    /**
     * Configures the alarm fired after crossing one of the limits (min / max
     * temperature, min / max humidity).
     *
     * @param device DeviceNumber, currently not supported.
     * @param alarm 0 = Alarm off 1 .. 9 = Number of Pieps on Alarm 10 = Infinit
     * Alarm, Manual stop on Humigraf.
     * @return 0 on success -1 Error establishing connection with device. -2
     * Error sending command to Humigraf -3 Error alarm not supportet
     */
    int setAlarm(int device, int alarm);

    /**
     * Get the acctual configured alarm from the system.
     *
     * @param device DeviceNumber, currently not supported.
     * @return 0 = Alarm off 1 .. 9 = Number of Pieps on Alarm 10 = Infinit
     * Alarm, Manual stop on Humigraf. -1 Error establishing connection with
     * device. -2 Error sending command to Humigraf -3 Error reading recieved
     * value
     */
    int getAlarm(int device);

    /**
     * If anything breaks while reading memory from the device after a
     * requestMemoryDump this will stop the command.
     *
     * @param device DeviceNumber currently not supportet.
     * @return 0 on success. -1 Error establishing connection with device. -2
     * Error sending command to Humigraf -3 Error reading recieved value
     */
    int abort(int device);

    /**
     * Format the Humigraf. It will completely format the device and instandly
     * capture a new pair of values.
     *
     * @param device DeviceNumber currently not supportet.
     * @return 0 if successfull
     */
    int deleteMemory(int device);

    /**
     * Get Devices as a String[] where dev[0] is empty and the rest consist of
     * devices
     *
     * @return List of Devices
     */
    String[] getDevices();

    /**
     * Get temperature & humidity from the Humigraf. The data will be recorded
     * instandly.
     *
     * @param device DeviceNumber, currently not supported.
     * @return double array with [0] humidity, [1] temperature
     */
    double[] getAcctualValues(int device);

    /**
     * Get the amount of memory used to store data elements.
     *
     * @return int value with amount of memory used. Represented by number of
     * bytes. -3 Unable to read, -2 Error sending, -1 error opening
     */
    int getMemorySize();

    /**
     * Sends a request to the Humigraf to start deliver data packages. Data
     * packages can be recieved by fireing getDataPackage(). The amount of
     * packages can be callculatet in advance with getMemorySize().
     *
     * @see getDataPackage()
     * @see getMeorySize()
     * @return 0 on success
     */
    int requestMemoryDump();

    /**
     * Get a data package from the Humigraf. This is only possible a
     * requestMemoryDump has been successfull in advance.
     *
     * @see requestMemoryDump
     * @return The packages will at most contain 32 values. Formatet by temp,
     * hum, temp, hum, where temp, hum go together. null on failure.
     */
    double[] getDataPackage();

    /**
     * @deprecated use requestMemoryDump() Recieve one hugh block of all
     * available data on the Humigraf. Was important for Version 2 of Humigraf.
     * @param device DeviceNumber, currently not supported.
     * @return hugh list of temperature & humidity. following the muster temp,
     * hum, temp, hum where allways one of each belong to the next
     */
    double[] readMemory(int device);

    /**
     * Get the lower temperature alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @return lower temperature alarm level or -1 on connection error, -2 on
     * sending error, -3 on reading error
     */
    int getMinTemp(int device);

    /**
     * Set the lower temperature alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @param temperature temperature for alarm level (Celsius)
     * @return 0 on success, -1 on connection error, -2 on sending error, -3 on
     * writing error
     */
    int setMinTemp(int device, int temperature);

    /**
     * Get the upper temperature alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @return upper temperature alarm level or -1 on connection error, -2 on
     * sending error, -3 on reading error
     */
    int getMaxTemp(int device);

    /**
     * Set the upper temperature alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @param temperature temperature for alarm level (Celsius)
     * @return 0 on success, -1 on connection error, -2 on sending error, -3 on
     * writing error
     */
    int setMaxTemp(int device, int temperature);

    /**
     * Get the lower humidity alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @return lower humidity alarm level or -1 on connection error, -2 on
     * sending error, -3 on reading error
     */
    int getMinHum(int device);

    /**
     * Set the lower humidity alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @param humidity humidity for alarm level (Celsius)
     * @return 0 on success, -1 on connection error, -2 on sending error, -3 on
     * writing error
     */
    int setMinHum(int device, int humidity);

    /**
     * Get the upper humidity alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @return upper humidity alarm level or -1 on connection error, -2 on
     * sending error, -3 on reading error
     */
    int getMaxHum(int device);

    /**
     * Set the upper humidity alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @param humidity humidity for alarm level (Celsius)
     * @return 0 on success, -1 on connection error, -2 on sending error, -3 on
     * writing error
     */
    int setMaxHum(int device, int humidity);

    /**
     * Get the upper humidity alarm level from Humigraf.
     *
     * @param device DeviceNumber, currently not supported.
     * @return 0 = empty,1 = weak, 2 = good, -1 on connection error, -2 on
     * sending error, -3 on writing error
     */
    int getBatteryState(int device);

    /**
     * Get the Sector in which the paper of the device operates
     *
     * @param device DeviceNumber, currently not supported
     * @return 0 = Standard (0...100% / -5...45°C) 1 = Reduziert (20...80% /
     * 5...35°C) -1 on connection error, -2 on sending error, -3 on writing
     * error
     */
    int getDeviceSector(int device);
}
